#!/bin/sh
#
# Component: checkintegrity
#
# Description:
#      Iterates through studies to be checked and calls CheckStudy for each study.
#      Studies that are OK will be added to '.done' list.
#      Studies having integrity errors tried to be fixed (if recover option is turned on).
#      If a study could be fixed it will be added to the '.fixed' list.
#      Studies cannot be fixed will be added to the '.error' list
#
# Usage:
#      checkAll.sh [-d] [-r] [-c <config file>] [-C <config setting>]...  -f <listfile>
#
#       -d                   : print debug messages
#       -r                   : recover mode
#       -f <listfile>        : contains study instance UIDs of the studies to be checked
#       -c <config file>     : configuration file to be used instead of var/checkintegrity/integrity.conf
#       -C <config setting>  : config setting to be added to the settings set in the configuration file
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh


LOGDIR=$TMP 

STUDIES_FILE=""
RECOVER=""
DEBUG=""
CONFIGFILE_OPT=""
CONFIGSETTING_OPT=""
while getopts 'f:c:C:rd' OPT; do
	case $OPT in
		f) STUDIES_FILE="$OPTARG" ;;
		c) CONFIGFILE_OPT="-c \"$OPTARG\"" ;;
		C) CONFIGSETTING_OPT="$CONFIGSETTING_OPT -C \"$OPTARG\"" ;;
		r) RECOVER="-r" ;;
		d) DEBUG="-d" ;;
		*)  ;;
	esac
done
	
if [ -z "$STUDIES_FILE" ] ; then
	echo "Missing Argument: File with Study Instance UIDs to be checked" >&2
	exit 1
fi

if [ ! -f "$STUDIES_FILE" ] ; then
	echo "$STUDIES_FILE does not exist" >&2
	exit 1
fi

TOCHECK_FILE="${STUDIES_FILE}"
FIXD_FILE="${STUDIES_FILE}.fixed"
DONE_FILE="${STUDIES_FILE}.done"
SKIP_FILE="${STUDIES_FILE}.error"
TEMP_FILE="${STUDIES_FILE}.temp"

MaxTry=3
counter=0

while true; do
	if [ $counter -ne 0 ] ; then
		sleep 10
		TOCHECK_FILE="${RCHK_FILE}"
		echo "Recovery mode: Re-check studies in $TOCHECK_FILE"
	fi

	let counter=counter+1
	RCHK_FILE="${STUDIES_FILE}.recheck.$counter"

	while [ `cat $TOCHECK_FILE | wc -l` -gt 0 ] ; do
		study=`head -n 1 $TOCHECK_FILE`
		echo "Checking $study ..."
		
		$TASKD_HOME/runjavacmd -c "0 imagemed.checkintegrity.CheckStudy -s $study $CONFIGFILE_OPT $CONFIGSETTING_OPT -l $LOGDIR/$study.log $RECOVER $DEBUG" -p $JAVA_SERVER_PORT
		RET=$?
		
		cat $LOGDIR/$study.log
		rm -f $LOGDIR/$study.log
		
		echo $RET
		if [ $RET -eq 0 ] ; then
			# Adding study to the .done list
			echo "OK - $study"
			echo "$study" >> $DONE_FILE
		elif [ $RET -ge 124 ] ; then
			# Adding study to the .skip list
			echo "ERROR - $study"
			echo "$study" >> $SKIP_FILE
		elif [ -z "$RECOVER" ] ; then
			# Not in recovery mode, add study to error list
			echo "ERROR - $study :: Inconsistencies detected"
			echo "$study" >> $SKIP_FILE
		else
			# Recovery mode: study might be fixed: recheck later
			echo "CHANGED - $study :: Some inconsistencies has been fixed: re-check later"
			echo "$study" >> $RCHK_FILE
		fi

		tail -n +2 $TOCHECK_FILE > $TEMP_FILE
		mv $TEMP_FILE $TOCHECK_FILE
	done

	if [ -z "$RECOVER" ] ; then
		echo "Check Only mode: exit"
		break
	fi

	if [ ! -f "$RCHK_FILE" ] ; then
		echo "No study was changed in recover mode[1]: exit"
		break
	fi

	if [ `cat $RCHK_FILE | wc -l` -eq 0 ] ; then
		echo "No study was changed in recover mode[2]: exit"
		break
	fi

	if [ $counter -eq $MaxTry ] ; then
		echo "Max try ($MaxTry) has been reached"
		break
	fi
done

echo
exit 0
