#!/bin/sh
#
# Component: checkintegrity
#
# Description:
#  If a .tar.gz file was not handed over than generates a .tar.gz file contains the output of
#  the objdiff.
#  If a .tar.gz file was handed over than unpack and run objdiff match the given parameters
#  and compare the two result.
#
# Usage:
#   See usage_() function below
#
# Exit Codes:
#   0 - OK
#   1 - An error occured
#  11 - Object file not found in study directory
#  21 - Differences in number of files extracted
#  22 - Differences in dcmdump files
#  23 - Differences in pixel data
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh


usage_()
{
	cat <<- ENDOFUSAGE
	compareobjinfo.sh { -c  <filename> | -t <filename> } [-i] -d <studydir> -o <objectfile>
	
		 -c <filename>      : the compressed file name contains the information compare the current result to
		 -t <filename>      : the target file name which will be a .tar.gz contains the result of the objdiff
		 -d <studydir>      : Studydir where the object is located
		 -o <objectfile>    : Object file name
		 -i                 : Ignore edit changes when comparing to compressed file (archived changes)
	
		If -c option given : it makes an objdiff dump in the studydir on object and compare the results.
				     Return 0 if everything is match.
		If -t option given : it makes and objdiff dump in the studydir on object and compress the result.
		
	ENDOFUSAGE
	exit 1
} # usage_


ignoreEditChanges="false"

while getopts 'c:t:d:o:i' OPT; do
	case $OPT in
		c) COMPARETO="$OPTARG" ;;
		t) TARGET="$OPTARG" ;;
		d) STUDYDIR="$OPTARG" ;;
		o) OBJECT="$OPTARG" ;;
		i) ignoreEditChanges="true" ;;
		*) usage_ ;;
	esac
done

if [ -n "$COMPARETO" ] && [ -n "$TARGET" ] ; then
	usage_
fi

if [ -z "$COMPARETO" ] && [ -z "$TARGET" ] ; then
	usage_
fi

if [ -z "$STUDYDIR" ] || [ -z "$OBJECT" ] ; then
	usage_
fi

if [ ! -f "$STUDYDIR/$OBJECT" ] ; then
	echo "The objectfile does not exist." >&2
	exit 11
fi

PBRFILENAME=""
if [ "${OBJECT#PbR.}" = "$OBJECT" ] ; then
	PBRFILENAME="`ls $STUDYDIR | grep ^PbR | tail -n 1`"
fi
COMMAND="$UTILS_HOME/bin/objdiff"

if [ -n "$PBRFILENAME" ] ; then
	COMMAND="$COMMAND -1 $STUDYDIR/$PBRFILENAME"
fi

CWD=`pwd`
WORK_DIR="$TMP/ci_objdiff.$$"
mkdir $WORK_DIR
cd $WORK_DIR
if [ $? -ne 0 ] ; then
	echo "Working directory $WORK_DIR could not be created" >&2
	exit 1
fi

# Cleans up working directory
# and exit
#
# [$1] exit code
cleanup_()
{
	cd $CWD
	rm -r "$WORK_DIR"
	exit $1
}


TARGET_SHORT_NAME="target"
TARGET_DIR="$WORK_DIR/$TARGET_SHORT_NAME"

if [ -n "$TARGET" ] ; then
	mkdir $TARGET_DIR
	if [ $? -ne 0 ] ; then
		echo "Target directory $TARGET_DIR could not be created" >&2
		cleanup_ 1
	fi
	
	$COMMAND -d $TARGET_DIR $STUDYDIR/$OBJECT > $TARGET_DIR/objdiff.out

	tar -czf $TARGET $TARGET_SHORT_NAME/*
	
elif [ -n "$COMPARETO" ] ; then
	
	if [ ! -f "$COMPARETO" ] ; then
		echo "The file does not exist which would compare to!" >&2
		cleanup_ 1
	fi

	# unpack the tar.gz received - it will create a target directory
	tar -xzf $COMPARETO
	if [ $? -ne 0 ] ; then
		cleanup_ 1
	fi
	
	LOCAL_DIR="$WORK_DIR/local"
	mkdir $LOCAL_DIR
	if [ $? -ne 0 ] ; then
		echo "Local target directory $LOCAL_DIR could not be created" >&2
		cleanup_ 1
	fi

	$COMMAND -d $LOCAL_DIR $STUDYDIR/$OBJECT > $LOCAL_DIR/objdiff.out

	if [ -n "$PBRFILENAME" -a "$ignoreEditChanges" = "true" ] ; then
		ignoreFile="$WORK_DIR/ignoretags"
		$DCMTK_HOME/bin/dcmdump +P f215,da31 $STUDYDIR/$PBRFILENAME | sed -n 's;^  *\((....,....)\).*;^ \*\1;p' > $ignoreFile
		grep -v -f $ignoreFile $TARGET_DIR/objdiff.out > $TARGET_DIR/objdiff.out.$$
		grep -v -f $ignoreFile $LOCAL_DIR/objdiff.out > $LOCAL_DIR/objdiff.out.$$
		mv $TARGET_DIR/objdiff.out.$$ $TARGET_DIR/objdiff.out
		mv $LOCAL_DIR/objdiff.out.$$ $LOCAL_DIR/objdiff.out
	fi

	if [ "`ls $TARGET_DIR | wc -l`" != "`ls $LOCAL_DIR | wc -l`" ] ; then
		echo "Difference in number of files" >&2
		cleanup_ 21
	fi

	for file in `ls $TARGET_DIR` ; do
		if [ "$file" == "objdiff.out" ] ; then
			diff $TARGET_DIR/$file $LOCAL_DIR/$file
			if [ $? -ne 0 ] ; then
				echo "Difference in dcmdump files" >&2
				cleanup_ 22
			fi
		else
			$UTILS_HOME/bin/objdiff -c $TARGET_DIR/$file $LOCAL_DIR/$file
			if [ $? -ne 0 ] ; then
				echo "Difference in pixel data $file" >&2
				cleanup_ 23
			fi
		fi
	done
fi

cleanup_ 0
